from src.RegexRiot import *
import unittest

class RiotOperationsString(unittest.TestCase):
    """
    These tests will check that the RiotStrings are forming correctly.
    Another set of tests will check wheter the RiotString match correctly.
    
    """

    def test_then(self):
        self.assertEqual(str(DIGIT.then(NON_ALPHANUM)), r'\d\W')
    
    def test_one_or_more(self):
        self.assertEqual(str(DIGIT.one_or_more()), r'\d+')
        self.assertEqual(str(riot('hello').one_or_more()), r'(hello)+')
        self.assertEqual(str(one_or_more(DIGIT)), r'\d+')
        self.assertEqual(str(one_or_more(riot('hello'))), r'(hello)+')

    def test_one_or_more_around_then(self):
        self.assertEqual(str(one_or_more(DIGIT.then(ALPHANUM))), r'(\d\w)+')



"""These tests are based on the exercises found on http://regextutorials.com/index.html

"""

from src.RegexRiot import *
import unittest
import re

class RegexTutorials(unittest.TestCase):

    def test_Floating_point_numbers(self):
        """
        This test is exercise 1 http://regextutorials.com/excercise.html?Floating%20point%20numbers

        Match numbers containing floating point. Skip those that don't.

        Speed of light in vacuum 299792458 m/s

        Standard atmosphere 101325 Pa
        
        Earth to sun distance 149600000 km
        
        Acceleration of gravity 9.80665 m/s^2
        
        Circumference to diameter ratio 3.141592
        
        Gas constant 8.3144621 J/mol*K

        """
        lines = [
                ["Speed of light in vacuum 299792458 m/s", []],
                ["Standard atmosphere 101325 Pa", []],
                ["Earth to sun distance 149600000 km", []],
                ["Acceleration of gravity 9.80665 m/s^2", ["9.80665"]],
                ["Circumference to diameter ratio 3.141592", ["3.141592"]],
                ["Gas constant 8.3144621 J/mol*K", ["8.3144621"]],
                ]
        pattern = one_or_more(DIGIT).then(DOT).then(one_or_more(DIGIT)).compile()
        for test, matchs in lines:
            self.assertEqual(
                pattern.findall(test),
                matchs
            )
    
    @unittest.expectedFailure
    def test_Years_before_1990(self):
        """
        This is exercise 2 http://regextutorials.com/excercise.html?Years%20before%201990

        Match titles of all films produced before 1990.

        1 The Shawshank Redemption (1994)

        2 The Godfather (1972)

        3 The Godfather: Part II (1974)

        4 Pulp Fiction (1994)
        
        5 The Good, the Bad and the Ugly (1966)

        6 The Dark Knight (2008)

        7 12 Angry Men (1957)

        8 Schindler's List (1993)

        9 The Lord of the Rings: The Return of the King (2003)

        10 Fight Club (1999)

        """
        lines = [
            ["1 The Shawshank Redemption (1994)", False],
            ["2 The Godfather (1972)", True],
            ["3 The Godfather: Part II (1974)", True],
            ["4 Pulp Fiction (1994)", False],
            ["5 The Good, the Bad and the Ugly (1966)", True],
            ["6 The Dark Knight (2008)", False],
            ["7 12 Angry Men (1957)", True],
            ["8 Schindler's List (1993)", False],
            ["9 The Lord of the Rings: The Return of the King (2003)", False],
            ["10 Fight Club (1999)", False],
        ]
        pattern = BEGINING.then(one_or_more(ANYTHING)).then(OPEN_PARENTHESIS) \
            .then(1).then(DIGIT).then(riot(0, to=8)).then(DIGIT) \
            .then(CLOSE_PARENTHESIS).compile()
        print(pattern)

        for l, m in lines:
            self.assertEqual(m, pattern.fullmatch(l) is not None)

    @unittest.expectedFailure
    def test_Hexadecimal_colors(self):
        """
        This test is exercise 3 http://regextutorials.com/excercise.html?Hexadecimal%20colors

        Match 24-bit hexadecimal colors. Skip 12 bit colors.

        24 bit:

        AliceBlue #F0F8FF

        AntiqueWhite #FAEBD7

        Aqua #00FFFF

        Aquamarine #7FFFD4

        Azure #F0FFFF

        12 bit:

        White #FFF

        Red #F00

        Green #0F0

        Blue #00F

        """
        lines = [
                ["24 bit:", []],
                ["AliceBlue #F0F8FF", ["#F0F8FF"]],
                ["AntiqueWhite #FAEBD7", ["#FAEBD7"]],
                ["Aqua #00FFFF", ["#00FFFF"]],
                ["Aquamarine #7FFFD4", ["#7FFFD4"]],
                ["Azure #F0FFFF", ["#F0FFFF"]],
                ["12 bit:", []],
                ["White #FFF", []],
                ["Red #F00", []],
                ["Green #0F0", []],
                ["Blue #00F", []],
            ]
        pattern = RiotString("#").compile()
        for test, matchs in lines:
            self.assertEqual(
                pattern.findall(test),
                matchs
            )



import re
import _operations

class RiotString:
    """
    This is the object that will hold and mutate regular expressions. 
    RiotStrings can be composed together to build more complicated regular 
    expressions. 
    
    For instance the expression ``\d+\.\d+`` can be built using RiotStrings as 
    ``one_or_more(DIGIT).then(DOT).then(one_or_more(DIGIT))``. Although this is 
    not as consice as writing the regex itself, with the help of autocomplete it 
    will be easier to write write regex as a chain of function calls that read
    that read out to english like text.
    
    """

    def __init__(self, a, b, combinator, unit) -> None:
        """Create a RiotString from a given expression. Use this to create a 
        building block from which you will build up to what ever complicated 
        regex you actually want. 
        Something like
        ``RiotString("Annie are you okay ").then("are you okay ").times(2).then("Annie")``
        to make the regex ``"Annie are you okay (are you okay ){2}Annie``
        
        """
        self._a = str(a)
        self._b = str(b)
        self._combinator = combinator
        self._unit = unit

    def __str__(self) -> str:
        return self._combinator(self._a, self._b)
    def __repr__(self) -> str:
        return f'<RiotString: "{str(self)}">'
    
    def then(self, rs):
        """
        The current RiotString followed by the next RiotString. 
        Use like so
        
        ``RiotString('a').then(RiotString('b')) => RiotString('ab')``

        Parameters
        ----------
        rs: The next RiotString or string

        """
        return RiotString(self,rs, _operations.then, False)

    def one_or_more(self):
        """
        The current RiotString repeated one or more times.
        Use like so

        ``one_or_more(RiotString('a')) => RiotString('a+')``
        or
        ``RiotString('a').one_or_more() => RiotString('a+')``

        """
        regex = _operations.one_or_more(str(self), self._unit)
        return RiotString(regex, "", lambda a,b: a, True)
    
    def compile(self) -> re.Pattern:
        """Return the compiled regex. This is the result of ``re.compile("pattern")``"""
        return re.compile(str(self))

one_or_more = RiotString.one_or_more

class RiotSet(RiotString):
    def __init__(self, *args, to=None) -> None:
        self.elements = [str(i) for i in args]
        self.to = str(to)
        if to is not None:
            assert len(args)==1, f"Got multiple arguments for set range start {str(args)}"
            a = f"[{self.elements[0]}-{self.to}]"
        else:
            a = f"[{''.join(self.elements)}]"
        
        super().__init__(a, "", lambda a,b: a, True)


def riot(seed, *args, to=None):
    """
    Simplified interface for RiotString
    """
    if not args and to is None:
        return RiotString(seed, "", lambda a,b: a, len(seed)==1)
    if not to is None:
        assert not args, f"Got extra argument for range start {args}"
        return RiotSet(seed, to=to)
    else:
        return RiotSet(seed, *args)




# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

import sys
import os
sys.path.append(os.path.join('..','src'))
sys.path.append(os.path.join('..'))

import pytomlpp
f = os.path.realpath(os.path.join(__file__,'..','..','..',"pyproject.toml"))
details = pytomlpp.load(f)


# project = 'RegexRiot'
project = details['project']['name']
copyright = '2024, Rahul Kandekar'
author = 'Rahul Kandekar'
# import re
# with(open(os.path.realpath(os.path.join(__file__,'..','..','..',"pyproject.toml")), 'r')) as f:
#     for ln in f.readlines():
#         if re.match(r'^version.*(\d+\.\d+\.\d+).*', ln):
#             match = re.match(r'^version.*(\d+\.\d+\.\d+).*', ln)
#             break
release = details['project']['version']

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ['sphinx.ext.autodoc', 'sphinx_copybutton']

templates_path = ['_templates']
exclude_patterns = []



# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

# html_theme = 'furo'
# html_theme = 'sphinx_rtd_theme'
html_theme = 'furo'
html_static_path = ['_static']
html_theme_options = {
    "footer_icons": [
        {
            "name": "GitHub",
            "url": "https://github.com/RK22000/regex-riot-python",
            "html": """
                <svg stroke="currentColor" fill="currentColor" stroke-width="0" viewBox="0 0 16 16">
                    <path fill-rule="evenodd" d="M8 0C3.58 0 0 3.58 0 8c0 3.54 2.29 6.53 5.47 7.59.4.07.55-.17.55-.38 0-.19-.01-.82-.01-1.49-2.01.37-2.53-.49-2.69-.94-.09-.23-.48-.94-.82-1.13-.28-.15-.68-.52-.01-.53.63-.01 1.08.58 1.23.82.72 1.21 1.87.87 2.33.66.07-.52.28-.87.51-1.07-1.78-.2-3.64-.89-3.64-3.95 0-.87.31-1.59.82-2.15-.08-.2-.36-1.02.08-2.12 0 0 .67-.21 2.2.82.64-.18 1.32-.27 2-.27.68 0 1.36.09 2 .27 1.53-1.04 2.2-.82 2.2-.82.44 1.1.16 1.92.08 2.12.51.56.82 1.27.82 2.15 0 3.07-1.87 3.75-3.65 3.95.29.25.54.73.54 1.48 0 1.07-.01 1.93-.01 2.2 0 .21.15.46.55.38A8.013 8.013 0 0 0 16 8c0-4.42-3.58-8-8-8z"></path>
                </svg>
            """,
            "class": "",
        },
    ],
}



from _riot import RiotString

DIGIT       = RiotString(r'\d', "", lambda a,b: a, unit=True)
'RiotString for a digit. ``\d``'
NON_DIGIT   = RiotString(r'\D', "", lambda a,b: a, unit=True)
'RiotString for non digit. ``\D``'
ANYTHING    = RiotString(r'.', "", lambda a,b: a, unit=True)
'RiotString to match any digit. ``.``'
ALPHANUM    = RiotString(r'\w', "", lambda a,b: a, unit=True)
'RiotString to match any alphanumeric character. ``\w``'
NON_ALPHANUM= RiotString(r'\W', "", lambda a,b: a, unit=True)
'RiotString to match any non-alphanumeric character. ``\W``'
SPACE       = RiotString(r'\s', "", lambda a,b: a, unit=True)
'RiotString to match any space character. ``\s``'
NON_SPACE   = RiotString(r'\S', "", lambda a,b: a, unit=True)
'RiotString to match any non-space character. ``\S``'
BOUNDARY    = RiotString(r'\b', "", lambda a,b: a, unit=True)
'RiotString to match a character at a boundary position. ``\\b``'
NON_BOUNDARY= RiotString(r'\B', "", lambda a,b: a, unit=True)
'RiotString to match a character not at a boundary position. ``\B``'
DOT         = RiotString(r'\.', "", lambda a,b: a, unit=True)
'RiotString to match a dot. ``\.``'
BEGINING    = RiotString(r'^', "", lambda a,b: a, unit=True)
'RiotString to match the begining of a line. ``^``'
END         = RiotString(r'$', "", lambda a,b: a, unit=True)
'RiotString to match the end of a line. ``$``'
OPEN_PARENTHESIS = RiotString(r'\(', "", lambda a,b: a, unit=True)
'RiotString to match an open round parenthesis . ``\(``'
CLOSE_PARENTHESIS = RiotString(r'\)', "", lambda a,b: a, unit=True)
'RiotString to match a close round parenthesis . ``\)``'






from src.RegexRiot import *
import unittest

class LiteralsStringValueTests(unittest.TestCase):
    
    def test_digit(self):
        self.assertEqual(str(DIGIT), r'\d')
        self.assertEqual(str(NON_DIGIT), r'\D')
    
    def test_anything(self):
        self.assertEqual(str(ANYTHING), r'.')
    
    def test_alphanum(self):
        self.assertEqual(str(ALPHANUM), r'\w')
        self.assertEqual(str(NON_ALPHANUM), r'\W')
        
    def test_space(self):
        self.assertEqual(str(SPACE), r'\s')
        self.assertEqual(str(NON_SPACE), r'\S')
    
    def test_boundary(self):
        self.assertEqual(str(BOUNDARY), r'\b')
        self.assertEqual(str(NON_BOUNDARY), r'\B')
    
    def test_dot(self):
        self.assertEqual(str(DOT), r'\.')
    
    def test_custom_string(self):
        words = ['hello','there', ',', ' ', 'general', 'kenobi']
        for w in words:
            self.assertEqual(str(riot(w)), w)
    
    def test_begining_and_end(self):
        self.assertEqual(str(BEGINING), r'^')
        self.assertEqual(str(END), r'$')
    
    def test_round_parentheses(self):
        self.assertEqual(str(OPEN_PARENTHESIS), r'\(')
        self.assertEqual(str(CLOSE_PARENTHESIS), r'\)')
        



# This block adds the RegexRiot directory onto the path
import os, sys
root = os.path.join(__file__, '..')
root = os.path.realpath(root)
sys.path.append(root)
del os, sys, root
#######################################################

from _basic_expressions import *
from _riot import *
